"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
exports.__esModule = true;
var async_csv_1 = require("async-csv");
var fs_1 = require("fs");
/**
 * Converts an array of objects into a CSV file.
 */
var ObjectsToCsv = /** @class */ (function () {
    /**
     * Creates a new instance of the object array to csv converter.
     * @param {object[]} objectArray
     */
    function ObjectsToCsv(objectArray) {
        this.data = [];
        if (!Array.isArray(objectArray)) {
            throw new Error('The input to objects-to-csv must be an array of objects.');
        }
        if (objectArray.length > 0) {
            if (objectArray.some(function (row) { return typeof row !== 'object'; })) {
                throw new Error('The array must contain objects, not other data types.');
            }
        }
        this.data = objectArray;
    }
    /**
     * Saves the CSV file to the specified file.
     * @param {string} filename - The path and filename of the new CSV file.
     * @param {object} options - The options for writing to disk.
     * @param {boolean} [options.append] - Whether to append to file. Default is overwrite (false).
     * @param {boolean} [options.bom] - Append the BOM mark so that Excel shows
     * @param {boolean} [options.allColumns] - Whether to check all items for column names or only the first.  Default is the first.
     * Unicode correctly.
     * @returns {string} The saved CSV string;
     */
    ObjectsToCsv.prototype.toDisk = function (filename, options) {
        return __awaiter(this, void 0, void 0, function () {
            var addHeader, fileNotExists, allColumns, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!filename) {
                            throw new Error('Empty filename when trying to write to disk.');
                        }
                        addHeader = false;
                        fileNotExists = !fs_1.existsSync(filename) || fs_1.statSync(filename).size === 0;
                        if (fileNotExists || !options || !options.append) {
                            addHeader = true;
                        }
                        allColumns = options && options.allColumns
                            ? options.allColumns
                            : false;
                        return [4 /*yield*/, this.toString(addHeader, allColumns)];
                    case 1:
                        data = _a.sent();
                        // Append the BOM mark if requested at the beginning of the file, otherwise
                        // Excel won't show Unicode correctly. The actual BOM mark will be EF BB BF,
                        // see https://stackoverflow.com/a/27975629/6269864 for details.
                        if (options && options.bom && fileNotExists) {
                            data = '\ufeff' + data;
                        }
                        if (options && options.append) {
                            return [2 /*return*/, new Promise(function (resolve, reject) {
                                    fs_1.appendFile(filename, data, 'utf8', function (error) {
                                        if (error) {
                                            reject(error);
                                        }
                                        else {
                                            resolve(data);
                                        }
                                    });
                                })];
                        }
                        else {
                            return [2 /*return*/, new Promise(function (resolve, reject) {
                                    fs_1.writeFile(filename, data, 'utf8', function (error) {
                                        if (error) {
                                            reject(error);
                                        }
                                        else {
                                            resolve(data);
                                        }
                                    });
                                })];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns the CSV file as string.
     * @param {boolean} header - If false, omit the first row containing the
     * column names.
     * @param {boolean} allColumns - Whether to check all items for column names.
     *   Uses only the first item if false.
     * @returns {Promise<string>}
     */
    ObjectsToCsv.prototype.toString = function (header, allColumns) {
        if (header === void 0) { header = true; }
        if (allColumns === void 0) { allColumns = false; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, convert(this.data, header, allColumns)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return ObjectsToCsv;
}());
/**
 * Private method to run the actual conversion of array of objects to CSV data.
 * @param {object[]} data
 * @param {boolean} header - Whether the first line should contain column headers.
 * @param {boolean} allColumns - Whether to check all items for column names.
 *   Uses only the first item if false.
 * @returns {string}
 */
function convert(data, header, allColumns) {
    if (header === void 0) { header = true; }
    if (allColumns === void 0) { allColumns = false; }
    return __awaiter(this, void 0, void 0, function () {
        var columnNames, csvInput;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (data.length === 0) {
                        return [2 /*return*/, ''];
                    }
                    columnNames = allColumns
                        ? __spread(data
                            .reduce(function (columns, row) {
                            Object.keys(row).map(function (rowKey) { return columns.add(rowKey); });
                            return columns;
                        }, new Set())) : Object.keys(data[0]);
                    if (allColumns) {
                        columnNames.sort(); // for predictable order of columns
                    }
                    csvInput = [];
                    if (header) {
                        csvInput.push(columnNames);
                    }
                    // Add all other rows:
                    csvInput.push.apply(csvInput, __spread(data.map(function (row) { return columnNames.map(function (column) { return row[column]; }); })));
                    return [4 /*yield*/, async_csv_1.stringify(csvInput)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports["default"] = ObjectsToCsv;
